/*------------------------------------------------------------------------------------
   Name: STS-Tape.mq4
   
   Description: Tape chart of the STS Matrix
	          
   Change log:
       2012-04-25. Xaphod, v1.02
          - Force repaint of all bars if newbars>3 on set TF
          - Moved color settings for 3 bars to input parameters for reinitialization 
            to work correctly
       2012-04-23. Xaphod, v1.01
          - Fixed Alerts
       2012-04-20. Xaphod, v1.00
          - First Release 
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright  2012, Xaphod"
#property link      "http://www.xaphod.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 DodgerBlue
#property indicator_color2 Red
#property indicator_width1  4
#property indicator_width2  4
#property indicator_width3  4
#property indicator_width4  4
#property indicator_maximum 1
#property indicator_minimum 0

// Win32 API
#import "user32.dll"
   bool     GetWindowRect(int hWnd, int& lpRect[]);
#import
#import "Kernel32.dll"
  int CreateFileA(string lpFileName,int dwDesiredAccess, int dwShareMode, int lpSecurityAttributes, int dwCreationDisposition,int dwFlagsAndAttributes, int hTemplateFile);
  int CloseHandle(int hObject);
#import

#define OPEN_EXISTING 3
#define FILE_SHARE_READ 1
#define GENERIC_READ 0x80000000
#define FILE_ATTRIBUTE_NORMAL 128
#define INVALID_HANDLE_VALUE 0xFFFFFFFF

// Constant definitions
#define INDICATOR_NAME "STS-Tape"
#define INDICATOR_VERSION "1.02"

#define IND_TVI        "TVI"
#define IND_GOLDMINER  "GoldMiner-Histo"
#define IND_BULLSBEARS "SEFC-Bull-Bear-c"

// Indicator parameters
extern string    Indi.Version=INDICATOR_VERSION;
extern string    TimeFrame.Settings="";
extern int       TimeFrame.Period=0;         // Timeframe: 0,1,5,15,30,60,240,1440 etc. Current Timeframe=0. 
extern int       TimeFrame.Auto=0;           // Automatically select higher TF. M15 and M30 -> H1. Off=0, 1st HTF=1, 2nd HTF=2
extern string    Mode.Settings="";
extern bool      Mode.Show3Bars=False;       // Indicate when 3 of 4 bars are the same
extern color     Mode.NoSigColor=DimGray;    // Color of no signal bar for 4 bars mode 
extern color     Mode.3UpColor=RoyalBlue;    // Color of up signal bar for 3 of 4 bars 
extern color     Mode.3DownColor=FireBrick;  // Color of down signal bar for 3 of 4 bars
extern string    TVI.Settings="";
extern int       TVI.r=12;
extern int       TVI.s=12;
extern int       TVI.u=5;
extern string    CCI.Settings="";
extern int       CCI.Period=20;
extern string    GoldMiner.Settings="";
extern int       GoldMiner.SSP=7;
extern double    GoldMiner.KMax=50.6;
extern string    BullsBears.Settings="";
extern int       BullsBears.Period=12;
extern bool      BullsBears.Classic=True;
extern string    BullsBears.Label="BB";
extern string    Alert.Settings="";
extern bool      Alert.OnBarClose=True;      // Alert only when an open bar closes
extern bool      Alert.Popup=False;          // Enable popup window & sound on alert
extern bool      Alert.Email=False;          // Enable send email on alert
extern string    Alert.Subject="";           // Email subject. Null string ("") will result in a preconfigured subject.

// Global module varables
// Histogram
double gadSigUp[];
double gadSigDn[];
double gadSigUpR[];
double gadSigDnR[];

double gadTVI[];
double gadCCI[];
double gadGM[];
double gadBB[];

// Globals
int giRepaintBars;
bool gbInit;

//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
  int iMaxTF;
  
  // Init indicator buffers
  IndicatorBuffers(8);
  // Tape Chart
  SetIndexStyle(0,DRAW_HISTOGRAM);
  SetIndexBuffer(0,gadSigUp);
  SetIndexStyle(1,DRAW_HISTOGRAM);
  SetIndexBuffer(1,gadSigDn);
  if (Mode.Show3Bars)
    SetIndexStyle(2,DRAW_HISTOGRAM,EMPTY,EMPTY,Mode.3UpColor);
  else
    SetIndexStyle(2,DRAW_HISTOGRAM,EMPTY,EMPTY,Mode.NoSigColor);
  SetIndexBuffer(2,gadSigUpR);
  SetIndexStyle(3,DRAW_HISTOGRAM,EMPTY,EMPTY,Mode.3DownColor);
  SetIndexBuffer(3,gadSigDnR);
  // Indicator data
  SetIndexStyle(4,DRAW_NONE);
  SetIndexBuffer(4,gadTVI);
  SetIndexStyle(5,DRAW_NONE);
  SetIndexBuffer(5,gadCCI);
  SetIndexStyle(6,DRAW_NONE);
  SetIndexBuffer(6,gadGM);  
  SetIndexStyle(7,DRAW_NONE);
  SetIndexBuffer(7,gadBB);
  // Null Label
  for (int i=0; i<8; i++) {
    SetIndexLabel(i,NULL);
    SetIndexEmptyValue(i,0.0);
  }
  
  // Set Timeframe
  switch(TimeFrame.Auto) {
    case 1: 
      TimeFrame.Period=NextHigherTF(TimeFrame.Period); 
      giRepaintBars=TimeFrame.Period/Period()+2;
    break;
    case 2: 
      TimeFrame.Period=NextHigherTF(NextHigherTF(TimeFrame.Period));
      giRepaintBars=TimeFrame.Period/Period()+2;
    break;
    default: 
      if (TimeFrame.Period<1 || TimeFrame.Period==Period()) {
        TimeFrame.Period=Period();
        giRepaintBars=0;
      }
      else {
        giRepaintBars=TimeFrame.Period/Period()+2;
      }
    break;
  }
  
  // Check if external indicators are installed
  if (!IndicatorExists(IND_TVI+".ex4"))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\indicators\\"+IND_TVI+".ex4 cannot be found.");
  if (!IndicatorExists(IND_GOLDMINER+".ex4"))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\indicators\\"+IND_GOLDMINER+".ex4 cannot be found.");
  if (!IndicatorExists(IND_BULLSBEARS+".ex4"))
    Alert(Symbol()+", "+TF2Str(Period())+", "+INDICATOR_NAME+": Error! "+TerminalPath()+"\\experts\\indicators\\"+IND_BULLSBEARS+".ex4 cannot be found.");
  
  // Misc
  IndicatorShortName(INDICATOR_NAME+"("+TF2Str(TimeFrame.Period)+")");
  //IndicatorDigits(0);
  gbInit=True;
  return(0);
}


//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
  return (0);
}


//-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars;
  int iCountedBars; 
  int i;
  
  // Get unprocessed bars
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  
  // Set bars to redraw
  if (NewBars(TimeFrame.Period)>3)
    iNewBars=Bars-1;
  else
    iNewBars=Bars-iCountedBars;
  if (iNewBars<giRepaintBars)
    iNewBars=giRepaintBars;
  
  // Calc Data
  ProcessTVI(iNewBars, TVI.r, TVI.s, TVI.u, gadTVI);
  ProcessCCI(iNewBars,CCI.Period, gadCCI);
  ProcessBullsBears(iNewBars,BullsBears.Period, BullsBears.Classic, gadBB);
  ProcessGoldMiner(iNewBars,GoldMiner.SSP, GoldMiner.KMax, gadGM);
  
  //Update Tape Chart
  for (i=iNewBars+TimeFrame.Period/Period()*GoldMiner.SSP;i>=0;i--) {
    if (UpSignal(i)==4) {
      gadSigUp[i]=1;
      gadSigDn[i]=0;
      gadSigUpR[i]=0;
      gadSigDnR[i]=0;
    }
    else if (DnSignal(i)==4) {
      gadSigUp[i]=0;
      gadSigDn[i]=1;
      gadSigUpR[i]=0;
      gadSigDnR[i]=0;
    }
    else if (UpSignal(i)==3 && Mode.Show3Bars) {
      gadSigUp[i]=0;
      gadSigDn[i]=0;
      gadSigUpR[i]=1;
      gadSigDnR[i]=0;
    }
    else if (DnSignal(i)==3 && Mode.Show3Bars) {
      gadSigUp[i]=0;
      gadSigDn[i]=0;
      gadSigUpR[i]=0;
      gadSigDnR[i]=1;
    }
    else if (!Mode.Show3Bars) {
      gadSigUp[i]=0;
      gadSigDn[i]=0;
      gadSigUpR[i]=1;
      gadSigDnR[i]=0;
    }
    else {
      gadSigUp[i]=0;
      gadSigDn[i]=0;
      gadSigUpR[i]=0;
      gadSigDnR[i]=0;
    }
  }
  
  // Alerts
  CheckAlert();

  // Clear Flags 
  if (gbInit) 
    gbInit=False;
  
  return(0);
}
//+------------------------------------------------------------------+


//-----------------------------------------------------------------------------
// function: NewBars()
// Description: Return nr of new bars on a TF
//-----------------------------------------------------------------------------
int NewBars(int iPeriod) {
  static int iPrevSize=0;
  int iNewSize;
  int iNewBars;
  datetime tTimeArray[];
  
  ArrayCopySeries(tTimeArray,MODE_TIME,Symbol(),iPeriod);
  iNewSize=ArraySize(tTimeArray);
  iNewBars=iNewSize-iPrevSize;
  iPrevSize=iNewSize;
  return(iNewBars);
}


//-----------------------------------------------------------------------------
// function: ProcessTVI()
// Description: Calc TVI data and update matrix
//-----------------------------------------------------------------------------
int ProcessTVI(int iNewBars, int r, int s, int u, double& vdSig[]) {
  int i,j;
  double dTVI0, dTVI1;
  int iHTFBar=-1;
  
  for (i=iNewBars;i>=0;i--) {
    // Get index for higher timeframe bar
    if (TimeFrame.Period>Period())
      j=iBarShift(Symbol(), TimeFrame.Period, Time[i]);
    else
      j=i;
    // Calc TVI  
    if (iHTFBar!=j) {
      iHTFBar=j;
      dTVI1=iCustom(Symbol(),TimeFrame.Period,IND_TVI,r,s,u,0,j+1);
      dTVI0=iCustom(Symbol(),TimeFrame.Period,IND_TVI,r,s,u,0,j);
    }
    // Set TVI status
    if (dTVI0==EMPTY_VALUE || dTVI1==EMPTY_VALUE)
      vdSig[i]=0;
    else if (dTVI0>=dTVI1)
      vdSig[i]=1;
    else  
      vdSig[i]=-1;
  }
  return(0);
}


//-----------------------------------------------------------------------------
// function: ProcessCCI()
// Description: Calc CCI data and update matrix
//-----------------------------------------------------------------------------
int ProcessCCI(int iNewBars, int iPeriod, double& vdSig[]) {
  int i,j;
  double dSig;
  int iHTFBar=-1;
  
  for(i=iNewBars; i>=0; i--) {
    // Shift index for higher time-frame
    if (TimeFrame.Period>Period() )
      j=iBarShift(Symbol(), TimeFrame.Period, Time[i]);
    else
      j=i;
    // Calc CCI
    if (iHTFBar!=j) {
      iHTFBar=j;
      dSig=iCCI(NULL, TimeFrame.Period, iPeriod, PRICE_TYPICAL, j);
    }
    // Set CCI status
    if (iClose(NULL,TimeFrame.Period,i)==0)
      vdSig[i]=0;
    else if (dSig>=0)
      vdSig[i]=1;
    else  
      vdSig[i]=-1;
  }
  return(0);
}


//-----------------------------------------------------------------------------
// function: ProcessBullBears()
// Description: Calc BullBears data and update matrix
//-----------------------------------------------------------------------------
int ProcessBullsBears(int iNewBars, int iPeriod, bool bClassic, double& vdSig[]) {
  int i, j;
  double dUp, dDn;
  int iHTFBar=-1;
  
  for(i=MathMax(iNewBars,iPeriod); i>=0; i--) {
    // Shift index for higher time-frame
    if (TimeFrame.Period>Period())
      j=iBarShift(Symbol(), TimeFrame.Period, Time[i]);
    else
      j=i;
    // Calc BullsBears  
    if (iHTFBar!=j) {
      iHTFBar=j;  
      dUp=iCustom(Symbol(),TimeFrame.Period,IND_BULLSBEARS, iPeriod, bClassic,1,j);
      dDn=iCustom(Symbol(),TimeFrame.Period,IND_BULLSBEARS, iPeriod, bClassic,2,j);
    }
    // Set BullBears status
    if (dUp>0)
      vdSig[i]=1;
    else if (dDn<0)
      vdSig[i]=-1;
    else
      vdSig[i]=0;
  }
  
  return(0);
}


//-----------------------------------------------------------------------------
// function: ProcessGoldMiner()
// Description: Calc GoldMiner data and update matrix
//-----------------------------------------------------------------------------
int ProcessGoldMiner(int iNewBars,int iSSP, double dKMax, double& vdSig[]) {
  int i,j,iUp;
  int iHTFBar=-1;
  
  for(i=iNewBars+TimeFrame.Period/Period()*GoldMiner.SSP; i>=0; i--) {
    // Shift index for higher time-frame
    if (TimeFrame.Period>Period())
      j=iBarShift(Symbol(), TimeFrame.Period, Time[i]);
    else
      j=i;    
    // Calc goldminer data
    if (iHTFBar!=j) {
      iHTFBar=j;
      iUp = iCustom(NULL,TimeFrame.Period,IND_GOLDMINER,GoldMiner.SSP,GoldMiner.KMax,0,j);
    }
    // Set GoldMiner status
    if (iUp==EMPTY_VALUE)
      vdSig[i]=0;
    else if (iUp==1)
      vdSig[i]=1;
    else  
      vdSig[i]=-1;
  }
  return(0);
}


//-----------------------------------------------------------------------------
// function: CheckAlert()
// Description: Check for new alerts
//-----------------------------------------------------------------------------
void CheckAlert() {
  static datetime tAlertBar;
  static int iPrevAlert=0;
  
  if (Alert.Popup || Alert.Email) {
    
    // Alert on the close of the current bar
    if (Alert.OnBarClose && tAlertBar<Time[iBarShift(Symbol(), TimeFrame.Period, Time[0])] && !gbInit) {
      
      tAlertBar=Time[iBarShift(Symbol(), TimeFrame.Period, Time[0])];
      // Clear alert flag
      if (iPrevAlert==1 && gadSigUp[1]!=1)
         iPrevAlert=0;
      else if (iPrevAlert==-1 && gadSigDn[1]!=1)
         iPrevAlert=0;
      // Alert and set alert flag
      if (gadSigUp[1]==1 && iPrevAlert!=1) {
         AlertNow(Symbol()+", "+TF2Str(Period())+": STS Buy Signal.");
         iPrevAlert=1;
      }  
      else if (gadSigDn[1]==1 && iPrevAlert!=-1) {
         AlertNow(Symbol()+", "+TF2Str(Period())+": STS Sell Signal.");
         iPrevAlert=-1;
      }
    }
  
    // Alert while the current bar is open
    if (!Alert.OnBarClose && tAlertBar<Time[iBarShift(Symbol(), TimeFrame.Period, Time[0])] && !gbInit) {
      // Clear alert flag
      if (iPrevAlert==1 && gadSigUp[0]!=1)
         iPrevAlert=0;
      else if (iPrevAlert==-1 && gadSigDn[0]!=1)
         iPrevAlert=0;      
      // Alert and set alert flag
      if (gadSigUp[0]==1 && iPrevAlert!=1) {
         AlertNow(Symbol()+", "+TF2Str(Period())+": STS Buy Signal.");
         iPrevAlert=1;
         tAlertBar=Time[iBarShift(Symbol(), TimeFrame.Period, Time[0])];
      }  
      else if (gadSigDn[0]==1 && iPrevAlert!=-1) {
         AlertNow(Symbol()+", "+TF2Str(Period())+": STS Sell Signal.");
         iPrevAlert=-1;
         tAlertBar=Time[iBarShift(Symbol(), TimeFrame.Period, Time[0])];
      }
    }
  }
}


//-----------------------------------------------------------------------------
// function: AlertNow()
// Description: Signal the popup and email alerts
//-----------------------------------------------------------------------------
void AlertNow(string sAlertMsg) {
  //Popup Alert 
  if (Alert.Popup) 
    Alert(INDICATOR_NAME,", ", sAlertMsg);      
  //Email Alert
  if (Alert.Email) {
    if (Alert.Subject=="")
      SendMail( sAlertMsg, "MT4 Alert!\n"+INDICATOR_NAME+"\n" + TimeToStr(Time[0],TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
    else 
      SendMail( Alert.Subject, "MT4 Alert!\n"+INDICATOR_NAME+"\n" + TimeToStr(Time[0],TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);          
  }
}


//-----------------------------------------------------------------------------
// function: UpSignal()
// Description: Return nr of up signals 
//-----------------------------------------------------------------------------
bool UpSignal(int i) {
  int j=0;
  
  if (gadTVI[i]==1) j++; 
  if (gadCCI[i]==1) j++;
  if (gadGM[i]==1) j++;
  if (gadBB[i]==1) j++;
  return(j);
}


//-----------------------------------------------------------------------------
// function: DnSignal()
// Description: Return nr of down signals 
//-----------------------------------------------------------------------------
bool DnSignal(int i) {
  int j=0;
  
  if (gadTVI[i]==-1) j++; 
  if (gadCCI[i]==-1) j++;
  if (gadGM[i]==-1) j++;
  if (gadBB[i]==-1) j++;
  return(j);
}


//-----------------------------------------------------------------------------
// function: IndicatorExists()
// Description: Check if an indicator exists
//-----------------------------------------------------------------------------
bool IndicatorExists(string sIndicatorName) {
  int hFile;
  string sFile;
    
  // Exit if dlls are disabled
  if (!IsDllsAllowed())
    return(True);
  
  // Try to open indicator 
  sFile=TerminalPath()+"\\experts\\indicators\\"+sIndicatorName;
  hFile=CreateFileA(sFile,0,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);  
  if (hFile==INVALID_HANDLE_VALUE) {
    return(False);    
  }
  else {
    CloseHandle(hFile);
    return(True);
  }
  return(False);
}


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+iPeriod);
  }
  return(0);
}


//-----------------------------------------------------------------------------
// function: NextHigherTF()
// Description: Select the next higher time-frame. 
//              Note: M15 and M30 both select H1 as next higher TF. 
//-----------------------------------------------------------------------------
int NextHigherTF(int iPeriod) {
  if (iPeriod==0) iPeriod=Period();
  switch(iPeriod) {
    case PERIOD_M1: return(PERIOD_M5);
    case PERIOD_M5: return(PERIOD_M15);
    case PERIOD_M15: return(PERIOD_H1);
    case PERIOD_M30: return(PERIOD_H1);
    case PERIOD_H1: return(PERIOD_H4);
    case PERIOD_H4: return(PERIOD_D1);
    case PERIOD_D1: return(PERIOD_W1);
    case PERIOD_W1: return(PERIOD_MN1);
    case PERIOD_MN1: return(PERIOD_MN1);
    default: return(Period());
  }
}